/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.registry.core.repository.dao;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.core.repository.bo.Endpoint;
import org.ow2.petals.registry.core.repository.hibernate.HibernateHelper;

import com.trg.search.Filter;
import com.trg.search.Search;
import com.trg.search.hibernate.HibernateSearchProcessor;

/**
 * @author Christophe HAMERLING - eBM WebSourcing
 * @author Charles Casadei - eBM WebSourcing
 * 
 */
public class EndpointDAO {

    private static Log logger = LogFactory.getLog(EndpointDAO.class);

    public EndpointDAO() {
        super();
    }

    @SuppressWarnings("unchecked")
    public List<Endpoint> query(HibernateHelper hibernateHelper, org.ow2.petals.registry.api.Query query)
            throws RegistryException {

        List<Endpoint> list = null;
        try {
            if (logger.isDebugEnabled()) {
                logger.debug("Query " + query);
            }

            HibernateSearchProcessor processor = HibernateSearchProcessor
                    .getInstanceForSessionFactory(hibernateHelper.getSessionFactory());
            Search search = new Search();
            search.setSearchClass(Endpoint.class);
            search.setDistinct(true);
            if (query.getEndpoint() != null) {
                search.addFilter(Filter.equal("name", query.getEndpoint().toString()));
            }

            if (query.getService() != null) {
                search.addFilterEqual("service", query.getService().toString());
            }

            if ((query.getInterface() != null)) {
                search.addFilterEqual("interfaceName", query.getInterface().toString());
            }

            if (query.getComponent() != null) {
                search.addFilter(Filter.equal("component", query.getComponent()));
            }

            if (query.getContainer() != null) {
                search.addFilter(Filter.equal("container", query.getContainer()));
            }

            if (query.getSubDomain() != null) {
                search.addFilter(Filter.equal("subdomain", query.getSubDomain()));
            }

            if (query.getType() != null) {
                search.addFilter(Filter.equal("type", query.getType()));
            }

            /*
             * if (!query.isIgnoreParams() && (query.getParams() != null) &&
             * (query.getParams().size() > 0)) {
             * logger.debug("Using query parameters"); Filter properties =
             * Filter.and(); Iterator<String> iter =
             * query.getParams().keySet().iterator(); while (iter.hasNext()) {
             * String key = iter.next(); Filter f = Filter
             * .some("properties.values", Filter.and(Filter.equal("clef", key),
             * Filter .equal("value", query.getParams().get(key))));
             * properties.add(f); } search.addFilter(properties); }
             */     
            search.setDistinct(false);
            list = processor.search(hibernateHelper.getSession(), search);
        } catch (RuntimeException ex) {
            logger.warn("Query problem", ex);
        }

        return list;
    }

    public String getId(Session session, String path) {
        String result = null;
        String query = "from Endpoint edp WHERE edp.path = :path";
        Query q = session.createQuery(query);
        q.setString("path", path);
        Endpoint e = (Endpoint) q.uniqueResult();
        if (e != null) {
            result = e.getId();
        }
        return result;
    }

    public Endpoint get(Session session, String path) throws DAOException {
        Endpoint result = null;

        if (logger.isDebugEnabled()) {
            logger.debug("Get endpoint from " + path);
        }

        final String query = "from Endpoint edp WHERE edp.path = '" + path + "'";
        try {
            Query q = session.createQuery(query);
            result = (Endpoint) q.uniqueResult();
        } catch (HibernateException e) {
            if (logger.isDebugEnabled()) {
                logger.debug("Error in the method get of EndpointDAO ", e);
            }
            throw new DAOException(e.getMessage());
        }

        return result;
    }

    @SuppressWarnings("unchecked")
    public List<Endpoint> getAllFromPath(Session session, String path) throws DAOException {
        List<Endpoint> list = null;

        if (logger.isDebugEnabled()) {
            logger.debug("Get all the endpoints under : " + path);
        }
        final String query = "from Endpoint ed WHERE ed.path like '" + path + "%'";

        if (logger.isDebugEnabled()) {
            logger.debug("Query = " + query);
        }

        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            list = session.createQuery(query).list();

            tx.commit();
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }

        return list;
    }

    @SuppressWarnings("unchecked")
    public List<Endpoint> getAllFromRegistry(Session session, String registry) throws DAOException {

        List<Endpoint> endpoint = null;
        String query = "from Endpoint ed where ed.rootRegistry ='" + registry + "'";
        try {
            Query q = session.createQuery(query);
            endpoint = q.list();
        } catch (HibernateException e) {
            if (logger.isDebugEnabled()) {
                logger.debug("Error in the method get of EndpointDAO ", e);
            }
            throw new DAOException(e.getMessage());
        }
        return endpoint;
    }

    public boolean endpointExists(Session session, String path) throws DAOException {
        return this.get(session, path) != null;
    }

    public void create(Session session, Endpoint endpoint) throws DAOException {
        Transaction tx = null;
        try {
            tx = session.beginTransaction();

            session.save(endpoint);
            tx.commit();
        } catch (final HibernateException e) {
            if (tx != null) {
                tx.rollback();
            }
            if (endpoint != null) {
                throw new DAOException("Error creating new " + endpoint.getClass().getName(), e);

            }
            throw new DAOException("Error creating new Object of type null", e);
        }
    }

    public void update(Session session, Endpoint endpoint, String path) throws DAOException {
        String id = this.getId(session, path);
        if (id != null) {
            endpoint.setId(id);
            this.merge(session, endpoint);
        } else {
            throw new DAOException("Can not find intial resource to update");
        }
    }

    public void merge(Session session, Endpoint endpoint) throws DAOException {
        Transaction tx = null;

        try {
            tx = session.beginTransaction();

            session.merge(endpoint);

            tx.commit();
        } catch (final Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            if (endpoint != null) {
                throw new DAOException("Error merging " + endpoint.getClass().getName(), e);
            }
            throw new DAOException("Error merging Object of type null", e);
        }
    }

    public long count(Session session, Class<Endpoint> endpoint) throws DAOException {
        try {
            final Query qr = session.createQuery("select count(*) from " + endpoint.getName());
            if (qr != null) {
                return ((Long) qr.uniqueResult()).longValue();
            } else {
                return -1;
            }
        } catch (final Exception e) {
            if (endpoint != null) {
                throw new DAOException("Error getting size " + endpoint.getName(), e);
            }
            throw new DAOException("Error getting size of type null", e);
        }
    }

    public int keepOnly(Session session, String registry) throws DAOException {
        int result = 0;

        final String query = "delete from Endpoint where rootRegistry != :rootRegistry";
        Query q = session.createQuery(query);
        q.setString("rootRegistry", registry);

        if (logger.isDebugEnabled()) {
            logger.debug("Query = '" + query + "' and rootregistry is " + registry);
        }

        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            result = q.executeUpdate();
            tx.commit();
            if (logger.isDebugEnabled()) {
                logger.debug(result + " entries have been deleted");
            }
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public boolean delete(Session session, String path) throws DAOException {
        boolean result = true;

        if (logger.isDebugEnabled()) {
            logger.debug("Delete the resource from " + path);
        }

        if ((path == null) || (path.trim().length() == 0)) {
            if (logger.isDebugEnabled()) {
                logger.debug("Can not delete an entry under null or empty path");
            }
            result = false;
        } else {
            if (path.endsWith("/")) {
                this.deletePath(session, path);
            } else {
                this.deleteEntry(session, path);
            }
        }
        return result;
    }

    public int deletePath(final Session session, final String path) throws DAOException {
        int result = 0;
        final String query = "delete from Endpoint ed WHERE ed.path like '" + path + "%'";
        Query q = session.createQuery(query);

        if (logger.isDebugEnabled()) {
            logger.debug("Query =" + query);
        }

        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            result = q.executeUpdate();
            tx.commit();

            if (logger.isDebugEnabled()) {
                logger.debug(result + " entries have been deleted");
            }
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }

        return result;
    }

    public List<Endpoint> getEndpointPath(Session session, String path) throws DAOException {
        List<Endpoint> result = null;

        final String query = "from Endpoint edp WHERE edp.path like '" + path + "%'";

        try {
            Query q = session.createQuery(query);
            // q.setParameter("path", path);
            result = q.list();
        } catch (HibernateException e) {
            if (logger.isDebugEnabled()) {
                logger.debug("Error in the method get of EndpointDAO ", e);
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public boolean deleteEntry(final Session session, final String path) throws DAOException {
        boolean result = true;

        if (logger.isDebugEnabled()) {
            logger.debug("Delete the entry from " + path);
        }

        // get then delete...
        Endpoint ed = null;
        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            ed = this.get(session, path);

            if (ed != null) {
                session.delete(ed);
                session.flush();
            }
            tx.commit();
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }

        return result;
    }

    public int deleteFrom(Session session, String registryName) throws DAOException {
        int result = 0;

        final String query = "delete from Endpoint where rootRegistry = :rootRegistry";
        Query q = session.createQuery(query);
        q.setString("rootRegistry", registryName);

        if (logger.isDebugEnabled()) {
            logger.debug("Query =" + query);
        }

        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            result = q.executeUpdate();
            tx.commit();
            if (logger.isDebugEnabled()) {
                logger.debug(result + " entries have been deleted");
            }
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }

    public int deleteAll(Session session) throws DAOException {
        int result = 0;

        final String query = "delete from Endpoint";

        if (logger.isDebugEnabled()) {
            logger.debug("Query =" + query);
        }

        Transaction tx = null;
        try {
            tx = session.beginTransaction();
            Query q = session.createQuery(query);
            result = q.executeUpdate();

            tx.commit();

            if (logger.isDebugEnabled()) {
                logger.debug(result + " entries have been deleted");
            }
        } catch (HibernateException e) {
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (Exception e2) {
                    throw new DAOException(e2.getMessage());
                }
            }
            throw new DAOException(e.getMessage());
        }
        return result;
    }
}
