/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.registry.core.repository.jdbc;

import java.io.File;
import java.io.IOException;

import org.hibernate.HibernateException;
import org.hibernate.SQLQuery;
import org.hibernate.Session;

import org.ow2.petals.registry.api.config.Constants;
import org.ow2.petals.registry.api.config.DBConfiguration;
import org.ow2.petals.registry.api.context.Context;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.core.repository.RepositoryLifecycle;
import org.ow2.petals.registry.core.repository.hibernate.HibernateHelper;

/**
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class JDBCRepositoryLifeCycle extends RepositoryLifecycle {

    /**
     * 
     */
    private static final String FILE = ":file:";

    private static final String HSQL_DRIVER = "org.hsqldb.jdbcDriver";

    /**
     * Creates a new instance of JDBCRepositoryLifeCycle
     * 
     * @param repository
     */
    public JDBCRepositoryLifeCycle(JDBCRepository repository) {
        super(repository);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.registry.core.repository.RepositoryLifecycle#doInit(java
     * .util.Map)
     */
    @Override
    protected void doInit(Context context) throws LifeCycleException {
        super.doInit(context);

        // create the embedded database if needed
        DBConfiguration config = context.getConfiguration().getDbConfiguration();
        if (this.log.isDebugEnabled()) {
            this.log.debug("Configuring Hibernate session with " + config.toString());
        }

        if (Constants.DEFAULT_DB_URL.equals(config.getUrl())) {
            // update the URL if file
            final String initialURL = config.getUrl();
            int end = initialURL.indexOf(FILE) + FILE.length();
            String prefix = initialURL.substring(0, end);
            String suffix = initialURL.substring(end, initialURL.length());
            if (!suffix.equals(context.getWorkspace().getRepositoryRootPath().getAbsolutePath())) {
                File path = new File(context.getWorkspace().getRepositoryRootPath(), suffix);
                if (!path.exists()) {
                    try {
                        path.createNewFile();
                    } catch (IOException e) {
                        throw new LifeCycleException(
                                "Can not create the database storage path under "
                                        + path.getAbsolutePath(), e);
                    }
                }
                String url = prefix + path.getAbsolutePath() + ";shutdown=true";
                if (this.log.isDebugEnabled()) {
                    this.log.debug("Updated HSQL URL is now '" + url + "'");
                }
                config.setUrl(url);
            }
        }

        HibernateHelper hibernateHelper = new HibernateHelper();
        hibernateHelper.configureSession(config);
        ((JDBCRepository) repository).setHibernateHelper(hibernateHelper);

        // FIXME : This is for HSQLDB, need to be abstracted...
        // set write delay...
        if (config.getDriver().startsWith(HSQL_DRIVER)) {
            Session session = null;
            try {
                session = hibernateHelper.getSession();
                SQLQuery query = session.createSQLQuery("SET WRITE_DELAY FALSE");
                query.executeUpdate();
            } catch (Exception e) {
                this.log
                        .warn("Can not set the write delay to 0, will pause problems on restart since data may not be deleted from database on stop...");
            } finally {
                if (session != null) {
                    try {
                        session.disconnect();
                    } catch (HibernateException e) {
                    }
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void doStop() throws LifeCycleException {
        HibernateHelper hibernateHelper = ((JDBCRepository) repository).getHibernateHelper();
        if (!hibernateHelper.getSessionFactory().isClosed()) {
            try {
                hibernateHelper.getSessionFactory().close();
            } catch (Exception e) {
                this.log.warn("Closing the session factory causes the following exception : "
                        + e.getMessage());
            }
        }
    }

}
